#
# This Perl script is sample code showing simple manipulation of NDS user 
# objects using the NetWare management files on the _admin volume.
#
# An indiviual user can be add or deleted from a prompt or a group of uses can 
# be added/deleted by specifying a file name, where each line of the file is 
# the name of a user to be added.
#
# Additionally, there is an option to add a trustee to a file/directory.
#
#	(C) Copyright 2001 Novell, Inc.
#	All Rights Reserved.

#use File::Path;

use strict;

#
# Set the datastream for a file handle to the passed in value.
#
sub SetDataStream(*$)
{
	my $fh = $_[0];
	my $dataStream = $_[1];

	my $result;
	my $command;
	my $error;

	$command = "<virtualIO><datastream name=\"$dataStream\"/></virtualIO>";
	seek $fh, 0, 0;
	if (!syswrite($fh, $command, length($command)))	
	{
		$result .= "Unable to send datastream command to NDS management.  ";
		seek $fh, 0, 0;
		if (sysread($fh, $error, 1000)) 
		{
			$result .= $error;
		}
		$result .= "\n";
	}
	return $result;
}

#
# Write a command to a file and get the result
#
sub WriteCommand(*$)
{
	my $fh = $_[0];
	my $command = $_[1];

	my $result;
	my $error;
	my $reply;

	seek $fh, 0, 0;
	if (!syswrite($fh, $command, length($command)))
	{
		$result .= "Unable to send command to virtual file.  ";
		seek $fh, 0, 0;
		if (sysread($fh, $error, 1000)) 
		{
			$result .= $error;
		}
		$result .= "\n";
	}
	else
	{
		seek $fh, 0, 0;
		sysread($fh, $reply, 1000);
		$result .= $reply;
	}
	return $result;
}

#
# Parse the result of adding an NDS user
#
sub ParseNDSResult($$)
{
	my $xml = $_[0];
	my $name = $_[1];
	my $addUser = "";
	my $removeUser = "";

	if ($xml =~ /<addUser>(.*)<\/addUser>/s)
	{
		$addUser = $1;
		if ($addUser =~ /<result value=\"(-?\d+)\">/s)
		{
			if ($1 == 0)
			{
				return "$name successfully added to NDS\n";
			}
			else
			{
				if ($1 == -606)
				{
					return "$name already exists as a user object in NDS\n";
				}
				else
				{
					return "Error $1 adding $name to NDS\n";
				}
			}
		}
	}
	elsif ($xml =~ /<removeUser>(.*)<\/removeUser>/s)
	{
		$removeUser = $1;
		if ($removeUser =~ /<result value=\"(-?\d+)\">/s)
		{
			if ($1 == 0)
			{
				return "$name successfully removed from NDS\n";
			}
			else
			{
				if ($1 == -601)
				{
					return "user $name not found in NDS\n";
				}
				else
				{
					return "Error $1 removing $name from NDS\n";
				}
			}
		}
	}
	else
	{
		return "Error parsing results of NDS operation\n";
	}
}

#
# Parse the result of the NSS operations
#
sub ParseNSSResult($$$)
{
	my $xml = $_[0];
	my $name = $_[1];
	my $file = $_[2];
	my $addUser = "";
	my $addQuota = "";

	if ($xml =~ /<addTrustee>(.*)<\/addTrustee>/s)
	{
		$addUser = $1;
		if ($addUser =~ /<result value=\"(-?\d+)\">/s)
		{
			if ($1 == 0)
			{
				return "Trustee $name successfully added to $file\n";
			}
			else
			{
				return "Error $1 adding trustee $name to $file\n";
			}
		}
	}
	else
	{
		return "Error parsing results of NSS operation\n";
	}
}

#
# Add a user
#
sub AddUser($$) 
{
	my $name = $_[0];
	my $ndsContext = $_[1];
	my $result;
	my $ret;
	my $surname;
	my $givenName;

	$result = "Processing user \"$name\"\n";
	#
	# Add a user to NDS
	#
	if (!($ret = SetDataStream(NDSFILE, "command")))
	{
		$result .= $ret;

		$name =~ /(\w*$)/;
		$surname = $1;

		$name =~ /(^\w*)/;
		$givenName = $1;

		my $ndsResult = WriteCommand(NDSFILE, 
					"<ndsRequest><ndsUser><addUser>".
					"<name>$name</name>".
					"<context>$ndsContext</context>".
					"<surname>$surname</surname>".
					"<fullName>$name</fullName>".
					"<givenName>$givenName</givenName>".
					"<userDescription>Auto added user</userDescription>".
					"<password>password</password>".
					"</addUser></ndsUser></ndsRequest>");
		$result .= ParseNDSResult($ndsResult, $name);
	}
	return $result;
}
	
#
# Remove a user
#
sub RemoveUser($$) 
{
	my $name = $_[0];
	my $ndsContext = $_[1];
	my $result;
	my $ret;

	$result = "Processing user \"$name\"\n";
	#
	# Remove a user from NDS
	#
	if (!($ret = SetDataStream(NDSFILE, "command")))
	{
		$result .= $ret;
		my $ndsResult = WriteCommand(NDSFILE, 
					"<ndsRequest><ndsUser><removeUser>".
					"<name>$name</name>".
					"<context>$ndsContext</context>".
					"</removeUser></ndsUser></ndsRequest>");
		$result .= ParseNDSResult($ndsResult, $name);
	}
	return $result;
}
	
#
# Add a trustee
#
sub AddTrustee($$$$) 
{
	my $name = $_[0];
	my $ndsContext = $_[1];
	my $rights = $_[2];
	my $fileName = $_[3];
	my $result;
	my $ret;
	my $surname;
	my $givenName;

	$result = "Adding trustee \"$name\" to file $fileName\n";
	#
	# Add a trustee to a file/dir
	#
	if (!($ret .= SetDataStream(NSSFILE, "command")))
	{
		$result .= $ret;
		my $nssResult = WriteCommand(NSSFILE, 
					"<nssRequest><authorizeNW><addTrustee>".
					"<context>$ndsContext</context>".
					"<fileName>$fileName</fileName>".
					"<name>$name</name>".
					"<rights>$rights</rights>".
					"</addTrustee></authorizeNW></nssRequest>");
		$result .= ParseNSSResult($nssResult, $name, $fileName);
	}
	return $result;
}

#
# Main
#
	my $action;
	my $name;
	my $fileName;
	my $rights;
	my $ndsContext;

	if ($#ARGV != 1 || $ARGV[0] eq "/?" || $ARGV[0] eq "-?") 
	{
		print "USAGE: users.pl server ndsContext\n".
			"Example: users server1 server1_tree\\context\n".
			"If running from a server use \".\" for the server name.";
		exit;
	}

	my $server = $ARGV[0];
	my $ndsContext = $ARGV[1];

	if ($server eq ".")
	{
		open(NSSFILE, "+<_admin:Manage_NSS\\manage.cmd") 
			or die "Error opening NSS management file ($!) on server";
		open(NDSFILE, "+<_admin:Manage_NSS\\nds.cmd") 
			or die "Error opening NDS management file ($!) on server";
	}
	else
	{
		open(NSSFILE, "+<\\\\$server\\_admin\\Manage_NSS\\manage.cmd") 
			or die "Error opening NSS management file ($!)";
		open(NDSFILE, "+<\\\\$server\\_admin\\Manage_NSS\\nds.cmd") 
			or die "Error opening NDS management file ($!) on server";
	}

MAIN: while(TRUE) 
{
	do 
	{
		print	"1. Create users from prompt\n".
				"2. Create users from a file\n".
				"3. Delete users from a prompt\n".
				"4. Delete users from file\n".
				"5. Add a trustee to a file\n".
				"6. exit\n".
				"Select action: ";
		$action = <STDIN>;
	}while ($action < 1 || $action > 6);

	if ($action == 1) 
	{
		for(;;)
		{
			print "Enter the user name (no name = exit): ";
			chomp($name = <STDIN>);
			if ($name eq "")
			{
				last;
			}
			print "\n---------------------------\n";
			my @results = AddUser($name, $ndsContext);
			print @results;
			print "---------------------------\n\n";
		}
	}
	elsif ($action == 2) 
	{
		print "Enter the file name for the list: ";
		$fileName = <STDIN>;
		open(LISTFILE, "<$fileName") or die "Error opening $fileName ($!)";
		print "\n---------------------------\n\n";
		while (chomp($name = <LISTFILE>))
		{
			my @results = AddUser($name, $ndsContext);
			print @results;
			print "\n";
		}
		print "---------------------------\n\n";
		close(LISTFILE);
	}
	elsif ($action == 3)
	{
		for(;;)
		{
			print "Enter the user name (no name = exit): ";
			chomp($name = <STDIN>);
			if ($name eq "")
			{
				last;
			}
			print "\n---------------------------\n";
			my @results = RemoveUser($name, $ndsContext);
			print @results;
			print "---------------------------\n\n";
		}
	}
	elsif ($action == 4) 
	{
		print "Enter the file name for the list: ";
		$fileName = <STDIN>;
		open(LISTFILE, "<$fileName") or die "Error opening $fileName ($!)";
		print "\n---------------------------\n\n";
		while (chomp($name = <LISTFILE>))
		{
			my @results = RemoveUser($name, $ndsContext);
			print @results;
			print "\n";
		}
		print "---------------------------\n\n";
		close(LISTFILE);
	}
	elsif ($action == 5) 
	{
		for(;;)
		{
			print "Enter the user name (no name = exit): ";
			chomp($name = <STDIN>);
			if ($name eq "")
			{
				last;
			}
			print "Enter the rights to assign (\"srwcemfa\"): ";
			chomp($rights = <STDIN>);
			print "Enter the file/dir (vol:filename): ";
			chomp($fileName = <STDIN>);
			print "\n---------------------------\n";
			my @results = AddTrustee($name, $ndsContext, $rights, $fileName);
			print @results;
			print "---------------------------\n\n";
		}
	}
	else
	{
		last MAIN;
	}
}
close(NDSFILE);
close(NSSFILE);
